/*
A simple program using OpenGL and GLUI--
	uses glReadPixels to figure out what objects are under the cursor.

Orion Sky Lawlor, olawlor@acm.org, 2007/01/18 (Public Domain)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glew.h> /* GL Extensions Wrangler, http://glew.sourceforge.net/ */
#include <GL/glut.h> /* GL Utilities Toolkit, http://freeglut.sourceforge.net/ */
#include <GL/glui.h> /* GL User Interface, http://glui.sourceforge.net/ */
#include "osl/vec4.h" /* GLSL-style C++ types, by Orion Lawlor */
#include "osl/mat4.h"
using osl::mat4;

/* Variables updated by the GUI */
float zoom=1.0, specExp=100.0;
int win_w, win_h; /**< Width and height (in pixels) of our window at last resize */

/* These are rotation matrices, updated by the GLUI_Rotation spheres */
mat4 modelMat(1.0), viewMat(1.0), light1Mat(1.0);

int do_thingy=0;

/**
  Draw everything in the world to the back buffer.
  
  If "forClick" is true, use object-ID colors.
  If "forClick" is false, use normal visible display colors.
*/
void generic_display(bool forClick) {
	// Clear the color and depth buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); /* actually clear color and depth */
	
	/* Depth testing */
	glEnable(GL_DEPTH_TEST);
	
	/* Alpha blending */
	glEnable(GL_BLEND);
	           /*  src color  + dest color  */
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	// Load the model and view matrices into OpenGL
	glLoadMatrixf(&viewMat[0][0]);
	glMultMatrixf(&modelMat[0][0]);

/* Draw all the objects in the world */
	if (forClick) glDisable(GL_LIGHTING); /* don't do lighting for click colors */

	/* Loop over my objects */
	enum {n_i=50};
	for (int i=0;i<n_i;i++) {
		if (forClick) glColor4ub(i,i>>8,234,255);
		else glColor4f(1,1,0,1);
		
		glutSolidTorus(0.2/n_i,0.1+0.6*i/n_i,4,6);
	}
	
	if (forClick) glColor4ub(255,255,3,255);
	else glColor4f(0,0,1,1);
	
	glutSolidSphere(0.07,40,60);
	
}


/* GLUT calls this routine every time our window needs to be redrawn */
void main_display(void) {
	generic_display(false);
	
	// Display the stuff drawn so far to the screen:
	glutSwapBuffers();
	
	// Leave the back buffer full of object IDs
	generic_display(true); //< avoid slow redraw every mouse movement
}

/* GLUT calls this routine every time our window changes size */
void main_reshape(int w, int h) {
	win_w=w; win_h=h;
	glViewport(0, 0, (GLsizei)win_w, (GLsizei)(win_h));
}

void main_mouse(int button,int state,int x,int y) {
	printf("Mouse is at %d, %d:\n",x,y);
	unsigned char pix[4];
	y=win_h-1-y; // <- danged OpenGL Y axis is upside-down from GLUT Y axis!
	glReadPixels(x,y,1,1, GL_RGBA,GL_UNSIGNED_BYTE, &pix[0]);
	printf("   Pixels: %d, %d, %d, %d\n",pix[0],pix[1],pix[2],pix[3]);
	int i=pix[0]+(pix[1]<<8);
	printf("   -> Object ID %d\n",i);
	//glutSwapBuffers();
}

int main(int argc, char** argv)
{
	glutInitWindowSize(800,600);
	glutInit(&argc,argv);

	// Ask for a back buffer, color buffer, and Z buffer in a window.
	int mode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
	glutInitDisplayMode (mode);
	int main_window = glutCreateWindow("GLUI Simple Demo");

	// Ask GLUT to call us when we need to redraw
	glutDisplayFunc (main_display);
	glutReshapeFunc (main_reshape);
	glutMouseFunc   (main_mouse);
	
	
	// Make some sliders (this is the only GLUI code you'll need!)
	GLUI *glui = GLUI_Master.create_glui_subwindow(main_window, 
		GLUI_SUBWINDOW_TOP);
	/* This causes the main window to be redrawn when a GLUI control changes */
	glui->set_main_gfx_window( main_window );

	(new GLUI_Spinner( glui, "Zoom:", &zoom))
		->set_float_limits(0.0, 10.0);
	(new GLUI_Spinner( glui, "SpecExp:", &specExp))
		->set_float_limits(0.0, 1000.0);
	new GLUI_Checkbox(glui, "Do Thingy",&do_thingy);
	
	new GLUI_Column(glui);
	new GLUI_Rotation(glui,"Model:",&modelMat[0][0]);
	new GLUI_Column(glui,false);
	new GLUI_Rotation(glui,"View:",&viewMat[0][0]);
	new GLUI_Column(glui,false);
	new GLUI_Rotation(glui,"Light:",&light1Mat[0][0]);
	
	// Process events until user closes our window...
	glutMainLoop();
	
	return 0;
}
